using System.Security.Authentication;
using System.Security.Cryptography.X509Certificates;
using FitConnect.Models;
using FitConnect.Services;
using FitConnect.Services.Models;
using Microsoft.Extensions.Logging;

namespace FitConnect;

/// <summary>
/// Fluent API for the FitConnect.Subscriber
/// </summary>
public class FluentSubscriber : Subscriber, IFluentApi<FluentSubscriber> {
    private OAuthAccessToken? token;

    public FluentSubscriber(FitConnectEndpoints endpoints,
        IOAuthService oAuthService,
        ISubmissionService submissionService,
        IRouteService routeService,
        X509Certificate2? certificate = null,
        ILogger? logger = null) : base(endpoints, certificate, oAuthService, submissionService,
        routeService, logger) {
    }

    /// <summary>
    /// 
    /// </summary>
    /// <param name="clientId"></param>
    /// <param name="clientSecret"></param>
    /// <param name="scope"></param>
    /// <returns></returns>
    public FluentSubscriber
        Authenticate(string clientId, string clientSecret, string? scope = null) {
        Owner.ClientId = clientId;
        Owner.ClientSecret = clientSecret;

        token ??= AuthenticateAsync(clientId, clientSecret, scope).Result;

        return this;
    }

    public FluentSubscriber GetSubmissions(string destinationId, out List<Submission> submissions) {
        submissions = new List<Submission>();
        throw new NotImplementedException();
    }

    public FluentSubscriber GetSubmission(string destinationId, string submissionId,
        out Submission submission) {
        throw new NotImplementedException();
    }

    /// <summary>
    /// 
    /// </summary>
    /// <param name="destinationId"></param>
    /// <param name="availableSubmissions"></param>
    /// <param name="submissions">Dictionary of destinationId</param>
    /// <returns></returns>
    /// <exception cref="InvalidOperationException"></exception>
    public FluentSubscriber PollSubmissions(string destinationId, out long availableSubmissions,
        out Dictionary<string, IEnumerable<string>> submissions) {
        if (token == null)
            throw new InvalidOperationException("You must call authenticate first.");

        var submissionsResult = SubmissionService.ListSubmissions(null, 0, 100).Result;
        availableSubmissions = submissionsResult.TotalCount;

        // Creating a dictionary of destinationId to submissionIds from the REST API result
        submissions = submissionsResult.Submissions
            .Select(s => new { s.DestinationId, s.SubmissionId })
            .ToDictionary(i => i.DestinationId,
                i =>
                    submissionsResult.Submissions
                        .Where(s => s.DestinationId == i.DestinationId)
                        .Select(d => d.SubmissionId));

        return this;
    }

    public FluentSubscriber GetAttachments(out object unknown) {
        throw new NotImplementedException();
    }

    public FluentSubscriber DecryptAttachments(out object o) {
        throw new NotImplementedException();
    }

    public FluentSubscriber DecryptData(out object o) {
        throw new NotImplementedException();
    }

    public FluentSubscriber DecryptMetadata(out object o) {
        throw new NotImplementedException();
    }

    public FluentSubscriber ConfirmSubmission() {
        throw new NotImplementedException();
    }
}
